'use strict';

// Entry point for extension-specific code

const SettingDefinitions = [
  {
    key: 'run_on_startup',
    title: 'Run dns leak test on startup',
    default_value: true,
    type: 'boolean'
  }, {
    key: 'dns_iterations',
    title: 'Number of DNS query iterations? (0=disable)',
    default_value: 3,
    type: 'number'
  }
];

const CONFIG_VERSION = 1;

// Helper method for adding context menu items, docs: https://github.com/airdcpp-web/airdcpp-apisocket-js/blob/master/GUIDE.md#addContextMenuItems
const { addContextMenuItems } = require('airdcpp-apisocket');

// Settings manager docs: https://github.com/airdcpp-web/airdcpp-extension-settings-js
const SettingsManager = require('airdcpp-extension-settings');

module.exports = async function (socket, extension, name) {
  const settings = SettingsManager(socket, {
    extensionName: extension.name,
    configFile: extension.configPath + 'config.json',
    configVersion: CONFIG_VERSION,
    definitions: SettingDefinitions,
  });

  //////////////////////////////////////////////////  //////////////////////////////////////////////

  const sendEventMessage = async (message) => {   
    // Send a single event message
    try {
      await socket.post('events', {
        // EXTENSION_VERSION is defined in the webpack config
        text: message,
        severity: 'info'
      });
    } catch (e) {
      console.error(`Failed to send test message: ${e.message}`);
    }
  };

  extension.onStart = async (sessionInfo) => {
    // Load saved settings
    await settings.load();

    // Run dns leak test on startup
    if (settings.getValue('run_on_startup')) { 
      startDNSLeakTest(settings.getValue('dns_iterations')); 
    }

    // Get dns iterations value
    if (settings.getValue('dns_iterations')) {
      // dnsIterations = setInterval(sendEventMessage, settings.getValue('dns_iterations');
    }

    // Add a test context menu item for the extension
    const subscriberInfo = {
      id: 'airdcpp-dns-leak-test',
      name: 'DNS leak test'
    };

    if (sessionInfo.system_info.api_feature_level >= 4) {
      addContextMenuItems(
        socket,
        [
          {
            id: 'DNS leak test',
            title: `Run DNS leak test`,
            icon: {
              semantic: 'pink bell' // See https://fomantic-ui.com/elements/icon.html for available Web UI icons
            },
            onClick: () => {
              const iterations = settings.getValue('dns_iterations');
              if(iterations > 0)
                startDNSLeakTest(iterations);
            },
            access: 'events_edit',
            filter: selectedIds => selectedIds.indexOf(extension.name) !== -1 // Add the menu item only for our own extension
          }
        ],
        'extension',
        subscriberInfo,
      );
    }
  };

  extension.onStop = () => {
    // Cleanup
    // When running the extension in unmanaged mode, it will not exit
    // if the server goes down (or the socket gets disconnected otherwise)
    //
    // Note that all added event listeners (and hooks) will be removed automatically when the socket gets disconnected
    // so you don't need to take care of those manually

  };

  ///////////////////////////////////////////////////////////////////////////////////////////////////

  const ipleak = require('./ipleak');

  async function startDNSLeakTest(numberOfRequests) {
    const ipleak_t = new ipleak.ipleaktest();

    // Start with displaying host ip information and Session Hash
    sendEventMessage(await ipleak_t.RequestHostIpInfo());
    sendEventMessage("🏁 Starting DNS leak test with session hash: " + ipleak_t.sessionHash);

    // Start request
    for (let request = 1; request <= numberOfRequests; request++) {
      sendEventMessage(`REQUEST-${request}`);
      let dns = await ipleak_t.RequestDNS();
      if (dns instanceof Error) { sendEventMessage(error.toString()); }
      dns.forEach((provider) => {
        sendEventMessage(" | 🌐 " + provider.ip + 
                         " | 🗃️ " + provider.dns_name + 
                         " | 📍 " + provider.country_name + "/" + provider.city_name);
      });
    }
    sendEventMessage("🛑 Ending DNS leak test...");
  }

};